<?php
App::uses('AppHelper', 'View/Helper');

App::uses('Menu', 'Navigation.Model');

/**
 * Navigation Helper
 *
 * To create a formatted menu pass the return from Menu::getMenu($id):-
 *
 * $this->Navigation->menu($menu, $attr = array());
 *
 * You can check the active state of an individual menu item:-
 *
 * $this->Navigation->checkActive($item);
 *
 * @author  Andy Carter
 * @package Navigation
 */
class NavigationHelper extends AppHelper {
	/**
	 * Helper dependencies
	 *
	 * @var array
	 * @access public
	 */
	public $helpers = array('Html');

	public $settings = array(
		'tag' => 'li',
		'wrapTag' => 'ul',
		'childWrapTag' => 'div',
		'childrenClass' => 'menu-item--has-children',
		'activeClass' => 'menu-item--active',
		'activeTrailClass' => 'menu-item--active-trail',
		'itemClassPrefix' => 'menu-item--',
		'model' => 'Menu',
		'depth' => null
	);

	public function __construct(View $View, $settings = array()) {

		parent::__construct($View, $settings);

		$this->settings = array_merge($this->settings, (array) $settings);

		return;

	}

	/**
	 * Returns a formatted menu
	 * @param  [type] $data [description]
	 * @param  array  $attr [description]
	 * @return [type]       [description]
	 */
	public function menu($data, $attr = array()) {

		// Set the depth of menu to return.
		$attr = array_merge($this->settings, (array) $attr);
		$depth = $attr['depth'];

		// Build up the menu with the active and active_trail flags set.
		$this->buildMenu($data, $depth, $attr);

		// Return the marked up menu.
		return $this->_formatMenu($data, $attr, true);

	}

	/**
	 * Build up the menu array with the correct active/active-trail flags set.
	 * @param  array 	$data  menu data
	 * @param  integer 	$depth depth to recursively run the method for
	 *                         reduces by 1 each run (set to null to
	 *                         build the entire menu)
	 * @param 	array 	$attr  settings data merged with passed overwritten parameters
	 * @param  integer  $activeParent	ID of parent of active item
	 * @return array
	 */
	public function buildMenu(&$data, $depth = null, $attr = array(), &$activeParent = false) {

		foreach ($data as &$item) {

			$hasChildren = isset($item['children']) && !empty($item['children']);

			if ($hasChildren && ($depth===null | $depth>1)) {

				$this->buildMenu(
					$item['children'],
					($depth===null ? null : $depth-1),
					$attr,
					$activeParent
				);

			} else {

				// We've reached the maximum depth of the current menu so
				// get rid of the children.
				unset($item['children']);

			}

			$item[$attr['model']]['active_trail'] = false;

			// Check for active states
			if ($this->checkActive($item, $attr['model'])) {

				$item[$attr['model']]['active'] = true;
				$activeParent = $item[$attr['model']]['parent_id'];

			} elseif ($activeParent == $item[$attr['model']]['id']) {

				$item[$attr['model']]['active_trail'] = true;
				$activeParent = $item[$attr['model']]['parent_id'];

			}

		}

		return $data;

	}


	/**
	 * Build up the menu recursively.
	 * @param  array   $data    array containing menu data
	 * @param  array   $attr
	 * @param  boolean $primary true on the first call, false thereafter to
	 *                          indicate child menu
	 * @return string  formatted menu
	 */
	protected function _formatMenu($data, $attr, $primary = false) {

		$return = '';

		if (!$primary && !empty($attr['childWrapTag'])) {

			$return .= "<{$attr['childWrapTag']}>";

		}

		$id = '';
		$classes = array();

		if ($primary && isset($attr['class']) && !empty($attr['class'])) {

			$classes[] = $attr['class'];

		}

		if ($primary && isset($attr['id']) && !empty($attr['id'])) {

			$id = " id='{$attr['id']}'";

		}

		$class = implode(' ', $classes);

		$return .= "<{$attr['wrapTag']}$id class='$class'>";

		foreach ($data as $item) {

			$hasChildren = isset($item['children']) && !empty($item['children']);

			$children = $hasChildren ? $this->_formatMenu($item['children'], $attr, false) : null;

			// Work out the classes to apply to the current menu item.

			$classes = array();

			if (isset($item[$attr['model']]['id']) && !empty($attr['itemClassPrefix'])) {

				$classes[] = $attr['itemClassPrefix'] . $item[$attr['model']]['id'];

			}

			if (isset($item[$attr['model']]['class']) && !empty($item[$attr['model']]['class'])) {

				$classes[] = $item[$attr['model']]['class'];

			}

			if ($hasChildren && !empty($attr['childrenClass'])) {

				$classes[] = $attr['childrenClass'];

			}

			if ($item[$attr['model']]['active'] && !empty($attr['activeClass'])) {

				$classes[] = $attr['activeClass'];

			}

			if ($item[$attr['model']]['active_trail'] && !empty($attr['activeTrailClass'])) {

				$classes[] = $attr['activeTrailClass'];

			}

			$class = implode(' ', $classes);

			// Output the menu item.

			$return .= "<{$attr['tag']} class='$class'>";

			$return .= $this->Html->link($item[$attr['model']]['name'], $item[$attr['model']]['url']);

			if ($hasChildren) {

				$return .= $children;

			}

			$return .= "</{$attr['tag']}>";

		}

		$return .= "</{$attr['wrapTag']}>";

		if (!$primary && !empty($attr['childWrapTag'])) {

			$return .= "</{$attr['childWrapTag']}>";

		}

		return $return;

	}

	/**
	 * Check for active state of a menu item. Used by _formatMenu() and can be
	 * called from within a view when wanting to play with the unprocessed
	 * menu array.
	 * @param  array $item 	menu item
	 * @param  string $menu_model the menu model as defined by settings or as overwritten by user
	 * @return boolean      true if active
	 */
	public function checkActive($item, $menu_model) {

		if (isset($item[$menu_model]['active']) && $item[$menu_model]['active']) {

			return true;

		} else {

			return $this->_checkActiveByRoute($item, $menu_model);

		}

	}

	/**
	 * Check the menu item against the current route.
	 * @param  array $item	menu item
	 * @param  string $menu_model the menu model as defined by settings or as overwritten by user
	 * @return boolean		true if there is a match
	 */
	protected function _checkActiveByRoute($item, $menu_model) {

		$isActive = false;

		// Check if the menu item's URL matches the current page.
		if (!$isActive && $item[$menu_model]['url']) {

			$isActive = Router::normalize($this->here) === Router::normalize($item[$menu_model]['url']);

		}

		// Check if the menu item's pattern matches the current page.
		if (!$isActive && $item[$menu_model]['pattern']) {

			$isActive = preg_match($item[$menu_model]['pattern'], Router::normalize($this->here));

		}

		return $isActive;

	}

}
